HTTP_HEADER = {
    'Host': '',
    'Connection': 'keep-alive',
    'Accept': 'application/json, text/plain, */*',
    'User-Agent': '',
    'Referer': 'https://www.bilibili.com',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7'
}

URL_UP_ALL_VIDEO = 'http://space.bilibili.com/ajax/member/getSubmitVideos'
URL_UP_CHANNEL = 'http://api.bilibili.com/x/space/channel/video'
URL_UP_ALBUM = 'http://api.bilibili.com/x/web-interface/view'
URL_UP_VIDEO = 'http://api.bilibili.com/x/player/playurl'
URL_UP_INFO = 'http://api.bilibili.com/x/space/acc/info'
URL_UP_CHANNELS = 'http://api.bilibili.com/x/space/channel/list'




